/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.stub;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.NoSuchElementException;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.randomizer.Randomizer;
import ru.ispras.testbase.TestBaseContext;
import ru.ispras.testbase.TestBaseQuery;
import ru.ispras.testbase.TestData;

import static ru.ispras.testbase.stub.Utils.*;

final class TdpImmRandom extends TdpBase {
  public static String NAME = "imm_random";
  public static int COUNT = 1;

  private int iteration = 0;

  private int min = 0;
  private int max = 0;
  private Map<String, Node> unknownImms = null;

  @Override
  boolean isSuitable(TestBaseQuery query) {
    return checkContextAttribute(query, TestBaseContext.TESTCASE, NAME);
  }

  @Override
  void initialize(TestBaseQuery query) {
    iteration = 0;
    min = getParameterAsInt(query, "min");
    max = getParameterAsInt(query, "max");
    unknownImms = extractUnknownImms(query);
  }

  @Override
  public boolean hasNext() {
    return unknownImms != null && iteration < COUNT;
  }

  @Override
  public TestData next() {
    if (!hasNext())
      throw new NoSuchElementException();

    final Map<String, Node> outputData = new LinkedHashMap<String, Node>();
    for (Map.Entry<String, Node> e : unknownImms.entrySet()) {
      final int value = Randomizer.get().nextIntRange(min, max);
      outputData.put(e.getKey(), NodeValue.newInteger(value));
    }

    iteration++;
    return new TestData(outputData);
  }
}
